@@ -8,16 +8,38 @@ from kombu import Connection, Producer
 from strongtyping.strong_typing import match_typing
 
 from kombunicator.MessageConsumer import MessageConsumer
+from kombunicator.BusPlugins import RPCRequestPlugin, CallbackPlugin
 
 
 class RPCClient:
 
     @match_typing
-    def __init__(self, connection_parameter: dict, request_key: str, default_callback: Union[FunctionType, MethodType] = None):
+    def __init__(self, connection_parameter: dict, request_key: str, default_callback: Union[FunctionType, MethodType] = None, plugin: str = None):
+        """Create a RPC Client to make request to another service. The message is routed via the RabbitMQ default exchange.
+
+        Parameters
+        ----------
+        connection_parameter : `dict`
+            Holds the connection parameters to a RabbitMQ instance.
+            - 'hostname': `str` Host where RabbitMQ is accessible
+            - 'virtual_host': `str` virtual host of RabbitMQ instance
+            - 'port': `int` port to RabbitMQ instance
+            - 'userid': `str` username
+            - 'password': `str` password
+        request_key: `str`
+            The request key as given by the RPC Server.
+        default_callback: `callable`
+            a default callback function for this RCPClient. Will be called if no function is specified during request.
+        plugin: `str`
+            Plugin for modifying the request and the callback based on an specific receiver message bus.
+
+        """
         self.connection_parameter = connection_parameter
         self.request_key = request_key
         self.default_callback = default_callback
 
+        self.plugin = plugin
+
     @property
     def _unique_string(self):
         return str(uuid.uuid4().hex)
@@ -66,7 +88,8 @@ class RPCClient:
             _thread_ready=_ready
         )
 
-        def _callback(payload, headers, properties):
+        @CallbackPlugin(plugin=self.plugin, message_correlation_id=message_correlation_id)
+        def _callback(payload, headers=None, properties=None):
             if properties['correlation_id'] == message_correlation_id:
                 reply_callback(payload, **kwargs)
                 self.consumer_thread.stop()
@@ -77,7 +100,11 @@ class RPCClient:
         self.consumer_thread.register_message_handler(_callback)
         self.consumer_thread.start()
         _ready.wait()
+        self._publish_request(message, result_queue_name=result_queue_name,
+                              message_correlation_id=message_correlation_id, headers=headers)
 
+    @RPCRequestPlugin()
+    def _publish_request(self, message, *, result_queue_name, message_correlation_id, headers=None):
         with Connection(**self.connection_parameter) as connection:
             with Producer(connection) as producer:
                 producer.publish(
